==> layer_sdk-0.9.347249-py3-none-any.whl: layer/exceptions/status_report.py <==
import json
import traceback
from abc import ABC
from os import path
from pathlib import Path
from traceback import FrameSummary
from typing import Any, List, Optional, Tuple


class ExecutionStatusReport(ABC):
    @property
    def message(self) -> str:
        return ""

    @property
    def cause(self) -> str:
        return ""


class SQLExecutionStatusReport(ExecutionStatusReport):
    def __init__(self, message: str, cause: str):
        self._message = message
        self._cause = cause

    @property
    def message(self) -> str:
        return self._message

    @property
    def cause(self) -> str:
        return self._cause

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, SQLExecutionStatusReport):
            return False
        return self.message == other.message and self.cause == other.cause


class FeaturesetExecutionStatusReport(ExecutionStatusReport):
    def __init__(self, reports: List[Tuple[str, ExecutionStatusReport]]):
        self._reports: List[Tuple[str, ExecutionStatusReport]] = reports

    @property
    def message(self) -> str:
        formatted = []
        for i in range(len(self._reports)):
            formatted.append(
                f"Build {self._reports[i][0]}: {self._reports[i][1].message}"
            )
        return "\n".join(formatted)

    @property
    def cause(self) -> str:
        formatted = []
        for i in range(len(self._reports)):
            formatted.append(
                f"Build {self._reports[i][0]}: {self._reports[i][1].cause}"
            )
        return "\n".join(formatted)

    @property
    def reports(self) -> List[Tuple[str, ExecutionStatusReport]]:
        return self._reports

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, FeaturesetExecutionStatusReport):
            return False
        return self._reports == other._reports


class GenericExecutionStatusReport(ExecutionStatusReport):
    def __init__(self, message: str):
        self._message = message

    @property
    def message(self) -> str:
        return self._message

    @property
    def cause(self) -> str:
        return ""

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, GenericExecutionStatusReport):
            return False
        return self.message == other.message


class PythonExecutionStatusReport(ExecutionStatusReport):
    def __init__(
        self,
        message: str,
        frames: List[FrameSummary],
        source_dir: Optional[Path] = None,
    ):
        if source_dir is not None:
            expanded_source_dir = path.expanduser(source_dir)
            self._message = self._strip_source_dir(message, expanded_source_dir)
            self._frames = [
                FrameSummary(
                    self._strip_source_dir(frame.filename, expanded_source_dir),
                    frame.lineno,
                    frame.name,
                    line=frame.line,
                )
                for frame in frames
            ]
        else:
            self._message = message
            self._frames = frames
        self._source_dir = source_dir

    @property
    def message(self) -> str:
        return self._message

    @property
    def frames(self) -> List[FrameSummary]:
        return self._frames

    @property
    def cause(self) -> str:
        return self._format_frame(self._frames[-1])

    @staticmethod
    def _format_frame(frame: FrameSummary) -> str:
        return f'File "{frame.filename}" on line {frame.lineno}: "{frame.line}"'

    @staticmethod
    def from_exception(
        exc: Exception, source_dir: Optional[Path] = None
    ) -> "PythonExecutionStatusReport":
        return PythonExecutionStatusReport(
            message=str(exc),
            frames=traceback.extract_tb(exc.__traceback__),
            source_dir=source_dir,
        )

    @staticmethod
    def _strip_source_dir(string: str, source_dir: str) -> str:
        return string.replace(source_dir + path.sep, "")

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, PythonExecutionStatusReport):
            return False
        return self.message == other.message and self.frames == other.frames


class ExecutionStatusReportFactory:
    @staticmethod
    def from_json(json_str: str) -> ExecutionStatusReport:
        payload = json.loads(json_str)
        report_type = payload["type"].lower()
        if report_type == "sql":
            return SQLExecutionStatusReport(payload["message"], payload["cause"])
        elif report_type == "python":
            message: str = payload["message"]
            frames: List[FrameSummary] = [
                FrameSummary(
                    filename=f["file"],
                    lineno=f["lineno"],
                    name=f["name"],
                    line=f["line"],
                )
                for f in payload["frames"]
            ]
            return PythonExecutionStatusReport(
                message=message,
                frames=frames,
            )
        elif report_type == "featureset":
            data = []
            for feature_data in payload["reports"]:
                build_id = feature_data["build_id"]
                report = ExecutionStatusReportFactory.from_json(
                    json.dumps(feature_data["report"])
                )
                data.append((build_id, report))
            return FeaturesetExecutionStatusReport(data)
        elif report_type == "generic":
            return GenericExecutionStatusReport(payload["message"])
        else:
            raise Exception("Invalid status report type")

    @staticmethod
    def to_json(report: ExecutionStatusReport) -> str:
        if isinstance(report, SQLExecutionStatusReport):
            dic_sql = {"type": "SQL", "message": report.message, "cause": report.cause}
            return json.dumps(dic_sql)
        elif isinstance(report, PythonExecutionStatusReport):
            dic_py = {
                "type": "Python",
                "message": report.message,
                "frames": [
                    {
                        "file": f.filename,
                        "line": f.line,
                        "lineno": f.lineno,
                        "name": f.name,
                    }
                    for f in report.frames
                ],
            }
            return json.dumps(dic_py)
        elif isinstance(report, FeaturesetExecutionStatusReport):
            reports = []
            for data in report.reports:
                report_json = ExecutionStatusReportFactory.to_json(data[1])
                reports.append({"build_id": data[0], "report": json.loads(report_json)})
            dic_fs = {"type": "Featureset", "reports": reports}
            return json.dumps(dic_fs)
        elif isinstance(report, GenericExecutionStatusReport):
            dic_generic = {"type": "Generic", "message": report.message}
            return json.dumps(dic_generic)
        else:
            raise Exception("Unsupported report type")

==> layer_sdk-0.9.350311-py3-none-any.whl: layer/exceptions/status_report.py <==
import json
import traceback
from abc import ABC
from os import path
from pathlib import Path
from traceback import FrameSummary
from typing import Any, List, Optional


class ExecutionStatusReport(ABC):
    @property
    def message(self) -> str:
        return ""

    @property
    def cause(self) -> str:
        return ""


class SQLExecutionStatusReport(ExecutionStatusReport):
    def __init__(self, message: str, cause: str):
        self._message = message
        self._cause = cause

    @property
    def message(self) -> str:
        return self._message

    @property
    def cause(self) -> str:
        return self._cause

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, SQLExecutionStatusReport):
            return False
        return self.message == other.message and self.cause == other.cause


class GenericExecutionStatusReport(ExecutionStatusReport):
    def __init__(self, message: str):
        self._message = message

    @property
    def message(self) -> str:
        return self._message

    @property
    def cause(self) -> str:
        return ""

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, GenericExecutionStatusReport):
            return False
        return self.message == other.message


class PythonExecutionStatusReport(ExecutionStatusReport):
    def __init__(
        self,
        message: str,
        frames: List[FrameSummary],
        source_dir: Optional[Path] = None,
    ):
        if source_dir is not None:
            expanded_source_dir = path.expanduser(source_dir)
            self._message = self._strip_source_dir(message, expanded_source_dir)
            self._frames = [
                FrameSummary(
                    self._strip_source_dir(frame.filename, expanded_source_dir),
                    frame.lineno,
                    frame.name,
                    line=frame.line,
                )
                for frame in frames
            ]
        else:
            self._message = message
            self._frames = frames
        self._source_dir = source_dir

    @property
    def message(self) -> str:
        return self._message

    @property
    def frames(self) -> List[FrameSummary]:
        return self._frames

    @property
    def cause(self) -> str:
        return self._format_frame(self._frames[-1])

    @staticmethod
    def _format_frame(frame: FrameSummary) -> str:
        return f'File "{frame.filename}" on line {frame.lineno}: "{frame.line}"'

    @staticmethod
    def from_exception(
        exc: Exception, source_dir: Optional[Path] = None
    ) -> "PythonExecutionStatusReport":
        return PythonExecutionStatusReport(
            message=str(exc),
            frames=traceback.extract_tb(exc.__traceback__),
            source_dir=source_dir,
        )

    @staticmethod
    def _strip_source_dir(string: str, source_dir: str) -> str:
        return string.replace(source_dir + path.sep, "")

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, PythonExecutionStatusReport):
            return False
        return self.message == other.message and self.frames == other.frames


class ExecutionStatusReportFactory:
    @staticmethod
    def from_json(json_str: str) -> ExecutionStatusReport:
        payload = json.loads(json_str)
        report_type = payload["type"].lower()
        if report_type == "sql":
            return SQLExecutionStatusReport(payload["message"], payload["cause"])
        elif report_type == "python":
            message: str = payload["message"]
            frames: List[FrameSummary] = [
                FrameSummary(
                    filename=f["file"],
                    lineno=f["lineno"],
                    name=f["name"],
                    line=f["line"],
                )
                for f in payload["frames"]
            ]
            return PythonExecutionStatusReport(
                message=message,
                frames=frames,
            )
        elif report_type == "generic":
            return GenericExecutionStatusReport(payload["message"])
        else:
            raise Exception("Invalid status report type")

    @staticmethod
    def to_json(report: ExecutionStatusReport) -> str:
        if isinstance(report, SQLExecutionStatusReport):
            dic_sql = {"type": "SQL", "message": report.message, "cause": report.cause}
            return json.dumps(dic_sql)
        elif isinstance(report, PythonExecutionStatusReport):
            dic_py = {
                "type": "Python",
                "message": report.message,
                "frames": [
                    {
                        "file": f.filename,
                        "line": f.line,
                        "lineno": f.lineno,
                        "name": f.name,
                    }
                    for f in report.frames
                ],
            }
            return json.dumps(dic_py)
        elif isinstance(report, GenericExecutionStatusReport):
            dic_generic = {"type": "Generic", "message": report.message}
            return json.dumps(dic_generic)
        else:
            raise Exception("Unsupported report type")
